@@ -5,6 +5,7 @@
 #' @importFrom stats runif
 "_PACKAGE"
 
-.onLoad = function(libname, pkgname) { # nocov start
+.onLoad = function(libname, pkgname) {
+  # nocov start
   backports::import(pkgname)
 } # nocov end
